"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/20 17:33:00
"""
from zjbbintest.Actuator.utils.string_dynamic_run import StringDynamicRun
from zjbbintest.bintest_data import BinTestData
from zjbbintest.Actuator.bin_test_exception.bin_test_exception import AssertFailException, BTAssertNotFoundException, BTAssertDictFormatException


class AssertDict:
    """
    常见断言字典
    """
    EQUALS = ["=", "==", "相等", "等于", "一致", "一样"]
    NOT_EQUALS = ["!=", "不等于", "不等", "不一致", "不一样"]
    GREATER = [">", "大于"]
    LESS = ["<", "小于"]
    GREATER_EQUALS = [">=", "大于等于"]
    LESS_EQUALS = ["<=", "小于等于"]
    IN = ["in", "包含", "属于"]
    NOT_IN = ["not in", "不包含", "不属于"]


class AssertExecutor:
    """
    断言执行器，执行单个断言
    """

    def __init__(self, expect_data, actual_data, assert_name):
        self.expect_data = expect_data
        self.actual_data = actual_data
        self.assert_name = assert_name

    def execute(self):
        """
        执行单个断言
        :return:
        """
        try:
            if self.assert_name in AssertDict.EQUALS:
                assert self.expect_data == self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                             f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.NOT_EQUALS:
                assert self.expect_data != self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                             f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.GREATER:
                assert self.expect_data > self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                            f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.LESS:
                assert self.expect_data < self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                            f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.GREATER_EQUALS:
                assert self.expect_data >= self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                             f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.LESS_EQUALS:
                assert self.expect_data <= self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                             f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.IN:
                assert self.expect_data in self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                             f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            elif self.assert_name in AssertDict.NOT_IN:
                assert self.expect_data not in self.actual_data, f"进行'{self.assert_name}'断言失败" \
                                                                 f"，期望值：{self.expect_data}，实际值：{self.actual_data}"
            else:
                try:
                    assert_res = BinTestData.assert_dict.get(self.assert_name)(self.expect_data, self.actual_data)
                    if not assert_res:
                        raise AssertFailException(self.expect_data, self.actual_data, self.assert_name)
                    else:
                        return assert_res
                except TypeError as e:
                    raise BTAssertNotFoundException(self.assert_name)
            return True
        except AssertionError as e:
            raise AssertFailException(self.expect_data, self.actual_data, self.assert_name)

    def __str__(self):
        """
        :return:
        """
        return f"expect: {self.expect_data}, actual: {self.actual_data}, assert: {self.assert_name}"

    def replace(self, case_var_dict, req_dict, resp_dict):
        """
        替换断言中的变量
        :param case_var_dict:
        :param req_dict:
        :param resp_dict:
        :return:
        """
        self.expect_data, _ = StringDynamicRun(self.expect_data, case_var_dict, req_dict, resp_dict).run()
        self.actual_data, _ = StringDynamicRun(self.actual_data, case_var_dict, req_dict, resp_dict).run()

    @staticmethod
    def create_obj_by_dict(assert_dict):
        """
        根据断言字典创建断言对象
        :param assert_dict:
        :return:
        """
        assert_name = assert_dict.get("operator")
        expect_data = assert_dict.get("expect")
        actual_data = assert_dict.get("actual")
        if expect_data is None or expect_data == "":
            raise BTAssertDictFormatException(assert_dict, "expect")
        if assert_name is None or assert_name == "":
            raise BTAssertDictFormatException(assert_dict, "operator")
        if actual_data is None or actual_data == "":
            raise BTAssertDictFormatException(assert_dict, "actual")
        return AssertExecutor(expect_data, actual_data, assert_name)


class BatchAssertExecutor:
    """
    批量断言执行器
    处理一个测试用例中一个步骤里的多条断言
    """

    def __init__(self, assert_list):
        self.__assert_list = assert_list

    def execute(self, case_var_dict, req_dict, resp_dict):
        """
        批量执行断言
        :return:
        """
        execute_res_list = []
        for assert_dict in self.__assert_list:
            # 1.创建单个断言执行对象
            assert_executor = AssertExecutor.create_obj_by_dict(assert_dict)
            # 2.替换变量
            assert_executor.replace(case_var_dict, req_dict, resp_dict)
            # 3.执行
            assert_res = assert_executor.execute()
            # 4.内容存放
            execute_res_list.append(assert_res)
        return all(execute_res_list)



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/5/9 10:26:00
"""
import configparser
import copy


def replace_values(json_obj, replace_dict):
    """
    替换json中的内容，替换
    :param json_obj:
    :param replace_dict:
    :return:
    """
    if isinstance(json_obj, dict):
        return {k: replace_values(v, replace_dict) for k, v in json_obj.items()}
    elif isinstance(json_obj, list):
        return [replace_values(element, replace_dict) for element in json_obj]
    else:
        return replace_dict.get(json_obj, json_obj)


def save_conf_var(ini_file):
    """
    根据ini文件，生成一个dict，用于替换json中的变量
    :param ini_file:
    :return:
    """
    conf_dict = {}
    config = configparser.ConfigParser()
    config.read(ini_file)
    for section in config.sections():
        conf_dict[section] = {}
        for key, value in config.items(section):
            conf_dict[section][key] = value
    return conf_dict


def generate_value_to_jsonpath_map(obj, path='$'):
    """
    根据json对象，生成一个dict,存放value和jsonpath的映射关系，key为value，value为jsonpath
    :param obj:
    :param path:
    :return:
    """
    value_to_path = {}

    def safe_key(key):
        # 如果键是数字或者以特殊字符开头，使用方括号和引号
        if key[0].isdigit() or not key.replace('_', '').isalnum():
            return f'["{key}"]'
        else:
            return key

    if isinstance(obj, dict):
        for key, value in obj.items():
            safe_key_str = safe_key(key)
            new_path = f'{path}.{safe_key_str}' if path != '$' else f'${safe_key_str}'
            if isinstance(value, (dict, list)):
                nested_paths = generate_value_to_jsonpath_map(value, new_path)
                for val, paths in nested_paths.items():
                    if val not in value_to_path:
                        value_to_path[val] = []
                    value_to_path[val].extend(paths)
            else:
                if value not in value_to_path:
                    value_to_path[value] = []
                value_to_path[value].append(new_path)

    elif isinstance(obj, list):
        for index, value in enumerate(obj):
            new_path = f'{path}[{index}]'
            if isinstance(value, (dict, list)):
                nested_paths = generate_value_to_jsonpath_map(value, new_path)
                for val, paths in nested_paths.items():
                    if val not in value_to_path:
                        value_to_path[val] = []
                    value_to_path[val].extend(paths)
            else:
                if value not in value_to_path:
                    value_to_path[value] = []
                value_to_path[value].append(new_path)

    return value_to_path


def get_req_bt_str_map(req_dict):
    """
    获取请求的bt_str map
    :param req_dict:
    :return:
    """
    res_jsonpath_map = {}
    for req_key, req_value in req_dict.items():
        if isinstance(req_value, dict):
            # 请求中，生成key和jsonpath的映射关系只需要考虑body和param
            new_req_value = {}
            if 'body' in req_value:
                new_req_value['body'] = req_value.get('body')
            if 'params' in req_value:
                new_req_value['params'] = req_value.get('params')
            res_jsonpath_map[int(req_key)] = generate_value_to_jsonpath_map(new_req_value)
    for req_id, req_jsonpath_info in res_jsonpath_map.items():
        for req_key, jsonpath_list in req_jsonpath_info.items():
            # 生成bt字符串
            bt_str_list = map(lambda req_jsonpath: f"$BT[var($req['{req_id}'].{req_jsonpath[1:]})]",
                              jsonpath_list)
            res_jsonpath_map[req_id][req_key] = list(bt_str_list)
    return res_jsonpath_map


def get_resp_bt_str_map(resp_dict):
    """
    获取响应的bt_str map
    :param resp_dict:
    :return:
    """
    resp_jsonpath_map = {}
    for resp_key, resp_value in resp_dict.items():
        if isinstance(resp_value, dict):
            # 响应中，生成key和jsonpath的映射关系需要考虑status_code和text，刚好resp_value只有这两😄
            resp_jsonpath_map[int(resp_key)] = generate_value_to_jsonpath_map(resp_value)
    for resp_id, resp_jsonpath_info in resp_jsonpath_map.items():
        for resp_key, jsonpath_list in resp_jsonpath_info.items():
            # 生成bt字符串
            bt_str_list = map(lambda resp_jsonpath: f"$BT[var($resp['{resp_id}'].{resp_jsonpath[1:]})]",
                              jsonpath_list)
            resp_jsonpath_map[resp_id][resp_key] = list(bt_str_list)
    return resp_jsonpath_map


def get_conf_bt_str_map(conf_dict):
    """
    获取配置的bt_str map
    :param conf_dict:
    :return:
    """
    conf_jsonpath_map = generate_value_to_jsonpath_map(conf_dict)
    for conf_key, jsonpath_list in conf_jsonpath_map.items():
        # 生成bt字符串
        # lambda表达式的作用是将$dev.ak转为$BT[var($conf.dev.ak)]
        bt_str_list = map(lambda conf_jsonpath: f"$BT[var($conf.{conf_jsonpath[1:]})]",
                          jsonpath_list)
        conf_jsonpath_map[conf_key] = list(bt_str_list)
    return conf_jsonpath_map


def case_json_replace_bintest_var(case_json, conf_bt_str_map, res_bt_str_map, resp_bt_str_map):
    """
    替换case_json中的变量
    :param case_json:
    :param conf_bt_str_map:
    :param res_bt_str_map:
    :param resp_bt_str_map:
    :return:
    """
    new_case = {
        'caseName': case_json.get('caseName'),
        'description': case_json.get('description'),
        'priority': case_json.get('priority'),
        'tags': []
    }
    new_case_steps = []
    case_steps = case_json.get('testSteps')
    pre_step_bt_str_map = copy.deepcopy(conf_bt_str_map)
    for step in case_steps:
        step_id = step.get('step')
        # 先处理pre_step_bt_str_map
        if step_id > 0:
            pre_step_bt_str_map = merge_dict(pre_step_bt_str_map,
                                             res_bt_str_map.get(step_id - 1)
                                             if res_bt_str_map.get(step_id - 1) else {})
            pre_step_bt_str_map = merge_dict(pre_step_bt_str_map,
                                             resp_bt_str_map.get(step_id - 1)
                                             if resp_bt_str_map.get(step_id - 1) else {})
        # print(pre_step_bt_str_map)
        request_json = step.get('request')
        # 这些内容只替换和conf的配置做替换
        new_request_json = {
            'url': request_json.get('url'),
            'path': request_json.get('path'),
            'method': request_json.get('method'),
            # 'params': request_json.get('params'),
            'headers': request_json.get('headers')
        }
        new_request_json = replace_nested_values(new_request_json, conf_bt_str_map)
        # param和body和所有做替换
        param_json = request_json.get('params')
        body_json = request_json.get('body')
        new_param_json = replace_nested_values(param_json, pre_step_bt_str_map)
        new_body_json = replace_nested_values(body_json, pre_step_bt_str_map)
        new_request_json['param'] = new_param_json
        new_request_json['body'] = new_body_json

        new_step = {
            'step': step_id,
            'request': new_request_json,
            'action': step.get('action'),
            'assert': step.get('assert')
        }
        new_case_steps.append(new_step)
    new_case['testSteps'] = new_case_steps
    return new_case


def merge_dict(dict1, dict2):
    """
    合并两个字典
    :param dict1:
    :param dict2:
    :return:
    """
    # 创建一个新的字典，用于存储合并后的键值对
    merged_dict = {}

    # 遍历两个字典的键值对，合并相同的键
    for k, v in dict1.items():
        # 如果这个键在第二个字典中也存在，则合并列表
        if k in dict2:
            merged_dict[k] = v + dict2[k]  # 合并两个列表
        else:
            merged_dict[k] = v  # 如果dict2中没有这个键，直接使用dict1中的值

    # 遍历第二个字典，添加dict1中不存在的键
    for k, v in dict2.items():
        if k not in merged_dict:
            merged_dict[k] = v

    return merged_dict


def replace_nested_values(original_dict, replacement_dict):
    """
    替换嵌套字典中的值
    :param original_dict:
    :param replacement_dict:
    :return:
    """
    if isinstance(original_dict, dict):
        new_dict = {}
        for key, value in original_dict.items():
            if isinstance(value, dict):
                new_dict[key] = replace_nested_values(value, replacement_dict)
            elif isinstance(value, list):
                new_dict[key] = [
                    replace_nested_values(item, replacement_dict) if isinstance(item, (dict, list)) else item for item
                    in value]
            elif value in replacement_dict:
                if value is None:
                    new_dict = None
                    break
                # 使用整个列表作为替换值
                # 如果replacement_dict[value]只有一个元素，则直接使用这个元素替换
                # 如果replacement_dict[value]有多个元素，则使用列表作为替换值，判断是否>10个元素，只取前5个和后5个元素
                new_values = None
                if len(replacement_dict[value]) == 1:
                    new_values = replacement_dict[value][0]
                elif len(replacement_dict[value]) >= 10:
                    new_values = replacement_dict[value][:5] + replacement_dict[value][-5:]
                else:
                    new_values = replacement_dict[value]
                new_dict[key] = {
                    value: new_values
                }
            else:
                new_dict[key] = value
        return new_dict
    elif isinstance(original_dict, list):
        return [replace_nested_values(item, replacement_dict) for item in original_dict]
    else:
        return original_dict


if __name__ == '__main__':
    # req_dict, resp_dict = parse_har_file(
    #     '/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/机器人搜索.har')
    # # print(req_dict)
    # # print(json.dumps(req_dict))
    # # print(json.dumps(resp_dict))
    # res_jsonpath_map = get_req_bt_str_map(req_dict)
    # print(json.dumps(res_jsonpath_map))
    # resp_jsonpath_map = get_resp_bt_str_map(resp_dict)
    # print(json.dumps(resp_jsonpath_map))
    # req_jsonpath_map = get_req_jsonpath_map(req_dict)
    # print(json.dumps(req_jsonpath_map))

    # conf_dict = save_conf_var('/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/test.ini')
    # # print(json.dumps(conf_dict))
    # conf_jsonpath_map = get_conf_bt_str_map(conf_dict)
    # # print(json.dumps(conf_jsonpath_map))
    #
    # case_json = analysis('/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/机器人搜索.har')
    # # print(json.dumps(case_json))
    #
    # new_case = case_json_replace_bintest_var(case_json, conf_jsonpath_map, res_jsonpath_map, resp_jsonpath_map)
    # print(json.dumps(new_case))
    pass



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/23 16:56:00
"""

# config_dict = {}
# func_dict = {}
# assert_dict = {}
# action_dict = {}
# run_case_list = []
# case_load_flag = False


class BinTestData:
    """
    BinTestData class
    """
    config_dict = {}
    func_dict = {}
    assert_dict = {}
    action_dict = {}
    run_case_list = []
    case_load_flag = False



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/10 16:00:00
"""
from jsonschema import validate, ValidationError

from zjbbintest.Actuator.bin_test_exception.bin_test_exception import FormatCheckException

case_json_schema = {
    "type": "object",
    "properties": {
        "caseName": {
            "type": "string"
        },
        "description": {
            "type": "string"
        },
        "priority": {
            "type": "integer",
        },
        "tags": {
            "type": "array",
            "items": {
                "type": "string"
            },
        },
        "testSteps": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "step": {
                        "type": "integer"
                    },
                    "request": {
                        "type": "object",
                        "properties": {
                            "url": {
                                "type": "string"
                            },
                            "path": {
                                "type": "string"
                            },
                            "method": {
                                "type": "string",
                                "enum": ["GET", "POST", "PUT", "DELETE", "HEAD",
                                         "OPTIONS", "PATCH", "CONNECT", "TRACE",
                                         "get", "post", "put", "delete", "head",
                                         "options", "patch", "connect", "trace"]
                            },
                            "headers": {
                                "type": "object"
                            },
                            "params": {
                                "type": "object"
                            },
                            "body": {
                                "type": "object"
                            }
                        },
                        "required": ["url", "path", "method"]
                    },
                    "action": {
                        "type": "array",
                        "items": {
                            "type": "string"
                        }
                    },
                    "assert": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "expect": {
                                    "type": ["string", "number", "object", "array", "boolean", "null", "integer"]
                                },
                                "actual": {
                                    "type": ["string", "number", "object", "array", "boolean", "null", "integer"]
                                },
                                "operator": {
                                    "type": "string"
                                }
                            },
                            "required": ["expect", "actual", "operator"]
                        }
                    }
                },
                "required": ["step", "request"]
            }
        }
    },
    "required": ["caseName", "priority", "testSteps"]
}


def check_case(data, case_name):
    try:
        validate(instance=data, schema=case_json_schema)
        return True
    except ValidationError as e:
        print("Validation failed.")
        print(f"Message: {e.message}")
        print(f"Schema path: {'.'.join(e.relative_schema_path)}")
        print(f"Instance path: {'.'.join(map(str, e.relative_path))}")
        raise FormatCheckException(case_name, e.context)



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/10 20:25:00
"""
from urllib.parse import urljoin
import requests
import json

from zjbbintest.Actuator.bin_test_exception.bin_test_exception import RequestException
from zjbbintest.Actuator.utils.string_dynamic_run import StringDynamicRun


class RequestObject:
    """
    将请求封装成一个对象，方便后续处理
    """

    def __init__(self, url, path, method, headers={}, params={}, body={}):
        self.url = url
        self.path = path
        self.method = method.upper()
        self.headers = convert_to_str(headers)
        self.params = convert_to_str(params)
        self.body = body

    @staticmethod
    def create_obj_by_dict(request_dict):
        """
        根据字典创建RequestObject对象
        :param request_dict:
        :return: RequestObject
        """
        url = request_dict.get("url")
        path = request_dict.get("path")
        method = request_dict.get("method")
        headers = convert_to_str(request_dict.get("headers"))
        params = convert_to_str(request_dict.get("params"))
        body = request_dict.get("body")
        return RequestObject(url, path, method, headers, params, body)

    def replace(self, case_var_dict, req_dict, resp_dict):
        """
        替换url、path、headers、params、body中的动态变量
        :return: dict
        """
        # 执行url中的动态变量
        executed_url, _ = StringDynamicRun(self.url, case_var_dict, req_dict, resp_dict).run()
        headers = self.headers
        # 执行headers中的动态变量
        executed_headers = {}
        for key, value in headers.items():
            executed_key, _ = StringDynamicRun(key, case_var_dict, req_dict, resp_dict).run()
            executed_value, _ = StringDynamicRun(value, case_var_dict, req_dict, resp_dict).run()
            executed_headers[executed_key] = executed_value
        params = self.params
        # 执行params中的动态变量
        executed_params = {}
        if params:
            for key, value in params.items():
                executed_key, _ = StringDynamicRun(key, case_var_dict, req_dict, resp_dict).run()
                executed_value, _ = StringDynamicRun(value, case_var_dict, req_dict, resp_dict).run()
                executed_params[executed_key] = executed_value
        body = self.body
        # 执行body中的动态变量
        executed_body = {}
        if body:
            for key, value in body.items():
                if isinstance(key, str):
                    executed_key, _ = StringDynamicRun(key, case_var_dict, req_dict, resp_dict).run()
                else:
                    executed_key = key
                if isinstance(value, str):
                    executed_value, _ = StringDynamicRun(value, case_var_dict, req_dict, resp_dict).run()
                else:
                    executed_value = value
                executed_body[executed_key] = executed_value
        self.url = executed_url
        self.path = self.path
        self.method = self.method
        self.headers = executed_headers
        self.params = executed_params
        self.body = executed_body

    def send(self, case_var_dict, req_dict, resp_dict):
        """
        发送请求，返回响应对象
        :return: ResponseObject
        """
        self.replace(case_var_dict, req_dict, resp_dict)
        url = urljoin(self.url, self.path)
        try:
            print(f"发起请求，请求信息为:{self}")
            response = requests.request(self.method, url, headers=self.headers,
                                        params=self.params, data=json.dumps(self.body))
        except Exception as e:
            raise RequestException(f"请求{self}失败，失败原因:{e}")
        response = ResponseObject(response.status_code, response.text, response.content, response.headers)
        print(f"请求成功，响应信息为:{response}")
        return response

    def __str__(self):
        # return f"RequestObject(url={self.url},path={self.path}," \
        #        f"method={self.method},headers={json.dumps(self.headers)}," \
        #        f"params={json.dumps(self.params)},body={json.dumps(self.body)}) "
        return f"""
{{
    "RequestObject" : {{
        "url": "{self.url}",
        "path": "{self.path}",
        "method": "{self.method}",
        "headers": {json.dumps(self.headers)},
        "params": {json.dumps(self.params)},
        "body": {json.dumps(self.body)}
    }}
}}
"""

    def obj_to_dict(self):
        """
        将对象转化为字典
        :return: dict
        """
        return {
            "url": self.url,
            "path": self.path,
            "method": self.method,
            "headers": self.headers,
            "params": self.params,
            "body": self.body
        }


class ResponseObject:
    """
    将响应封装成一个对象，方便后续处理
    """

    def __init__(self, status_code, text, content, response_headers):
        self.status_code = status_code
        self.text = text
        self.content = content
        self.response_headers = dict(response_headers)

    def __repr__(self):
        return f"""
            {{
                "ResponseObject": {{
                    "status_code": {self.status_code},
                    "text": "{self.text}",
                    "content": "{self.content}",
                    "response_headers": {json.dumps(self.response_headers)}
            }}
            """

    def __str__(self):
        # return f"ResponseObject(status_code={self.status_code},text={self.text}," \
        #        f"content={self.content},response_headers={json.dumps(self.response_headers)})"
        return f"""
            {{
                "ResponseObject": {{
                    "status_code": {self.status_code},
                    "text": "{self.text}",
                    "content": "{self.content}",
                    "response_headers": {json.dumps(self.response_headers)}
            }}
            """

    def obj_to_dict(self):
        """
        将对象转化为字典
        :return: dict
        """
        if isinstance(self.content, bytes):
            self.content = self.content.decode("utf-8")
        if isinstance(self.text, bytes):
            self.text = self.text.decode("utf-8")
        try:
            self.content = json.loads(self.content)
            self.text = json.loads(self.text)
        except Exception as e:
            print(f"解析响应内容失败，原因：{e}")
            pass
        return {
            "status_code": self.status_code,
            "text": self.text
            # "content": self.content,
            # "response_headers": self.response_headers
        }


def convert_to_str(dictionary):
    """
    将字典中的key和value都转化为字符串
    :param dictionary:
    :return:
    """
    if dictionary is None:
        return {}
    new_dict = {}
    for key, value in dictionary.items():
        new_key = str(key) if not isinstance(key, str) else key
        new_value = str(value) if not isinstance(value, str) else value
        new_dict[new_key] = new_value
    return new_dict


class Requestor:
    """
    请求器，用于发送请求
    """

    def send(self, request_dict):
        """
        发送请求，返回响应对象
        :param request_dict: 请求信息
        :return: ResponseObject
        """


if __name__ == '__main__':
    pass



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/21 16:56:00
"""
import logging

from zjbbintest.Actuator.bin_test_exception import bin_test_exception as bt_exception
from zjbbintest.Actuator.utils.action_execution import BatchActionExecutor
from zjbbintest.Actuator.utils.assert_execution import BatchAssertExecutor
from zjbbintest.Actuator.utils.format_check import check_case
from zjbbintest.Actuator.utils.requestor import RequestObject
from enum import Enum
import json


class CaseRunStatus(Enum):
    """
    case执行状态枚举，
    0：未执行，属于未执行状态
    200：case执行通过，属于成功状态
    500：case未执行通过，断言失败，属于失败状态
    501：请求异常，属于失败状态
    401：case格式检查异常，属于跳过状态
    402：case中的BT字符串格式异常，属于跳过状态
    403：case中bt-func未找到异常，属于跳过状态
    404：case中bt-action未找到异常，属于跳过状态
    405: case中bt-assert未找到异常，属于跳过状态
    406: case中bt-func格式检查异常，属于跳过状态
    407：case中bt-action格式检查异常，属于跳过状态
    408：case中bt-assert格式检查异常，属于跳过状态
    409：case的assert的dict格式存在问题，属于跳过状态
    410：case信息为空，属于跳过状态
    999：其他异常，属于跳过状态
    """
    NOT_RUNNING = 0
    PASS = 200
    FAIL = 500
    REQUEST_EXCEPTION = 501
    CASE_FORMAT_CHECK_ERROR = 401
    BT_STRING_FORMAT_ERROR = 402
    BT_FUNC_NOT_FOUND = 403
    BT_ACTION_NOT_FOUND = 404
    BT_ASSERT_NOT_FOUND = 405
    BT_FUNC_FORMAT_CHECK_ERROR = 406
    BT_ACTION_FORMAT_CHECK_ERROR = 407
    BT_ASSERT_FORMAT_CHECK_ERROR = 408
    ASSERT_DICT_FORMAT_ERROR = 409
    CASE_INFO_IS_EMPTY = 410
    OTHER_EXCEPTION = 999


class BinTestCase:
    """
    case的执行空间，case的执行依靠这个对象
    case空间的内容有几部分
    1.case_name: case名称
    2.case_desc: case详情
    3.case_steps: case执行步骤，主要逻辑都在里面
    4.case_var_dict: case执行过程中记录的变量
    """

    def __init__(self, case_name="", case_desc="", priority="", case_mack_labels=[], case_steps=[],
                 case_run_status=CaseRunStatus.NOT_RUNNING, case_run_msg="未执行",
                 case_var_dict={}, req_dict={}, resp_dict={}):
        self.case_name = case_name
        self.case_desc = case_desc
        self.priority = priority
        self.case_mack_labels = case_mack_labels
        self.case_steps = case_steps
        # 下面四个是在创建之初没有的，在执行过程中会添加进去
        self.case_run_status = case_run_status
        self.case_run_msg = case_run_msg
        self.case_var_dict = case_var_dict
        self.req_dict = req_dict
        self.resp_dict = resp_dict

    @staticmethod
    def create_obj_by_dict(case_space_dict):
        """
        根据字典创建caseSpace对象
        :param case_space_dict:
        :return:
        """
        # 1.格式检查
        case_name = case_space_dict.get("caseName") if case_space_dict.get("caseName") else "QAQ case名称不存在"
        try:
            check_case(case_space_dict, case_name)
        except bt_exception.FormatCheckException as e:
            case_run_status = CaseRunStatus.CASE_FORMAT_CHECK_ERROR
            case_run_msg = str(e.message)
            return BinTestCase(case_name=case_name, case_run_status=case_run_status, case_run_msg=case_run_msg)
        case_desc = case_space_dict.get("description")
        priority = case_space_dict.get("priority")
        case_mack_labels = case_space_dict.get("tags")
        case_steps = case_space_dict.get("testSteps")
        case_step_list = []
        for case_step in case_steps:
            case_step_obj = CaseStep.create_obj_by_dict(case_step)
            case_step_list.append(case_step_obj)
        return BinTestCase(case_name=case_name, case_desc=case_desc, priority=priority,
                           case_mack_labels=case_mack_labels, case_steps=case_step_list)

    def run(self):
        """
        执行case
        :return:
        """
        logging.debug(f"开始执行case: {self.case_name}")
        # 判断case是否可以执行
        if self.case_run_status != CaseRunStatus.NOT_RUNNING:
            logging.info(f"case: {self.case_name} 已经执行过了，不能重复执行")
            # 不是未执行状态，说明执行过（有问题），不能重复执行
            return
        # 初始化case_var_dict
        self.case_var_dict = {}
        # 请求和响应的list
        self.req_dict = {}
        self.resp_dict = {}
        case_step_run_res = {}
        # print(self.case_steps)
        exception_flag = False
        # 执行case步骤
        logging.debug(f"------------------开始执行case步骤-----------------------")
        try:
            for case_step in self.case_steps:
                # TODO 执行case步骤的时候，需要把case_var_dict传进去，也需要请求和响应的list，这里需要补充一下
                # DONE 2024-05-05
                run_res, self.case_var_dict, self.req_dict, self.resp_dict = case_step.run(self.case_var_dict,
                                                                                           self.req_dict,
                                                                                           self.resp_dict)
                logging.debug(f"case_step: {case_step.step_id} 执行结果为：{run_res}")
                logging.debug(f"case_var_dict: {self.case_var_dict}")
                logging.debug(f"req_list: {self.req_dict}")
                logging.debug(f"resp_list: {self.resp_dict}")
                case_step_run_res[case_step.step_id] = run_res

        except (bt_exception.AssertFailException, AssertionError) as e:
            self.case_run_status = CaseRunStatus.FAIL
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.RequestException as e:
            self.case_run_status = CaseRunStatus.REQUEST_EXCEPTION
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.FormatCheckException as e:
            self.case_run_status = CaseRunStatus.BT_STRING_FORMAT_ERROR
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTFuncNotFoundException as e:
            self.case_run_status = CaseRunStatus.BT_FUNC_NOT_FOUND
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTActionNotFoundException as e:
            self.case_run_status = CaseRunStatus.BT_ACTION_NOT_FOUND
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTAssertNotFoundException as e:
            self.case_run_status = CaseRunStatus.BT_ASSERT_NOT_FOUND
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTFuncFormatCheckException as e:
            self.case_run_status = CaseRunStatus.BT_FUNC_FORMAT_CHECK_ERROR
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTActionFormatCheckException as e:
            self.case_run_status = CaseRunStatus.BT_ACTION_FORMAT_CHECK_ERROR
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTAssertFormatCheckException as e:
            self.case_run_status = CaseRunStatus.BT_ASSERT_FORMAT_CHECK_ERROR
            self.case_run_msg = str(e.message)
            exception_flag = True
        except bt_exception.BTAssertDictFormatException as e:
            self.case_run_status = CaseRunStatus.ASSERT_DICT_FORMAT_ERROR
            self.case_run_msg = str(e.message)
            exception_flag = True
        except Exception as e:
            exception_flag = True
            logging.exception(e)
            self.case_run_status = CaseRunStatus.OTHER_EXCEPTION
            self.case_run_msg = str(e)
        if not exception_flag:
            if len(case_step_run_res.items()) == len(self.case_steps) \
                    and all(case_step_run_res.values()):
                self.case_run_status = CaseRunStatus.PASS
                self.case_run_msg = "执行成功"
            else:
                self.case_run_status = CaseRunStatus.FAIL
                self.case_run_msg = "执行失败"
        return {
            "case_name": self.case_name,
            "case_desc": self.case_desc,
            "priority": self.priority,
            "mark": self.case_mack_labels,
            "run_status": self.case_run_status.value,
            "run_msg": self.case_run_msg
        }


class CaseStep:
    """
    case单个步骤对象
    case单个步骤的内容有几部分
    1.请求:dict
    2.动作:list
    3.断言:list
    """

    def __init__(self, step_id, request_obj, action_list, assert_list):
        self.step_id = step_id
        self.request_obj = request_obj
        self.action_list = action_list
        self.assert_list = assert_list

    def __str__(self):
        # return f"CaseStep(request_obj={str(self.request_obj)}, " \
        #        f"action_list={str(self.action_list)}, " \
        #        f"assert_list={str(self.assert_list)})"
        return f"""
            {{
                "CaseStep": {{
                    "request_obj" : {str(self.request_obj)},
                    "action_list" : {json.dumps(self.action_list)},
                    "assert_list" : {json.dumps(self.assert_list)}
                }}
            }}"""

    @staticmethod
    def create_obj_by_dict(case_step_dict):
        """
        根据字典创建caseStep对象
        :param case_step_dict:
        :return:
        """
        step_id = case_step_dict.get("step")
        request_dict = case_step_dict.get("request")
        request_obj = RequestObject.create_obj_by_dict(request_dict)
        actions = case_step_dict.get("action")
        action_list = BatchActionExecutor(actions)
        asserts = case_step_dict.get("assert")
        assert_list = BatchAssertExecutor(asserts)
        return CaseStep(step_id, request_obj, action_list, assert_list)

    def run(self, case_var_dict, req_dict, resp_dict):
        """
        执行case步骤
        :return:
        """
        # 请求
        bt_resp = self.request_obj.send(case_var_dict, req_dict, resp_dict)
        # 此时，self.request_obj已经完成了其中变量的替换，可以使用obj_to_dict()将其转为字典
        req_dict[str(self.step_id)] = self.request_obj.obj_to_dict()
        resp_dict[str(self.step_id)] = bt_resp.obj_to_dict()
        # 动作
        case_var_dict = self.action_list.execute(case_var_dict, req_dict, resp_dict)
        # 断言
        # TODO 断言的执行也需要case_var_dict, req_list, resp_list
        # DONE 2024-05-05
        step_run_res = self.assert_list.execute(case_var_dict, req_dict, resp_dict)
        return step_run_res, case_var_dict, req_dict, resp_dict


if __name__ == '__main__':
    # request = RequestObject("https://aiob-open.baidu.com", "/aiob-server/api/v2/getToken", "POST",
    #                         {'Content-Type': 'application/json'}, {}, {
    #                             "accessKey": "dfc272b758fb451cb5d409d6bf04b40e",
    #                             "secretKey": "561f119cea874a37a81a1e83c9e1b92d"
    #                         })
    # request_dict = request.obj_to_dict()
    # print(request_dict)
    # step_dict = {}
    # step_dict["request"] = request_dict
    # step_dict["action"] = ["", ""]
    # step_dict["assert"] = [
    #     {
    #         "expect": 200,
    #         "actual": 200,
    #         "operator": "=="
    #     },
    #     {
    #         "expect": "success",
    #         "actual": "success",
    #         "operator": "=="
    #     },
    #     {
    #         "expect": {
    #             "key": "value"
    #         },
    #         "actual": {
    #             "key": "value"
    #         },
    #         "operator": "=="
    #     }
    # ]
    # json_data = json.dumps(step_dict)
    # step = CaseStep.create_obj_by_dict(step_dict)
    # print(step.run({}))
    json_data = """{
  "caseName": "测试案例1",
  "description": "这是测试案例描述",
  "priority": 0,
  "testSteps": [
    {
      "step": 1,
      "request": {
        "url": "https://aiob-open.baidu.com",
        "path": "/aiob-server/api/v2/getToken",
        "method": "POST",
        "headers": {
          "Content-Type": "application/json"
        },
        "param": {
          
        },
        "body": {
          "accessKey": "dfc272b758fb451cb5d409d6bf04b40e",
          "secretKey": "561f119cea874a37a81a1e83c9e1b92d"
        }
      },
      "action": [
        "",
        ""
      ],
      "assert": [
        {
          "expect": 200,
          "actual": 200,
          "operator": "=="
        },
        {
          "expect": "success",
          "actual": "success",
          "operator": "=="
        },
        {
          "expect": {
            "key": "value"
          },
          "actual": {
            "key": "value"
          },
          "operator": "=="
        }
      ]
    }
  ]
}"""
    data = json.loads(json_data)
    bt_case = BinTestCase.create_obj_by_dict(data)
    bt_case.run()
    print(bt_case.case_run_status)
    print(bt_case.case_run_msg)



import os

from jinja2 import Environment, FileSystemLoader


def render_report(cases, test_time, report_path):
    # 计算数据
    total_cases = len(cases)
    passed_cases = len([case for case in cases if case['run_status'] == 200])

    path = os.path.dirname(os.path.abspath(__file__))
    # 加载模板文件
    file_loader = FileSystemLoader(path)
    env = Environment(loader=file_loader)

    # 选择模板
    template = env.get_template('report_template.html')

    # 渲染模板
    output = template.render(cases=cases, test_time=test_time, total_cases=total_cases, passed_cases=passed_cases)

    # 将渲染后的HTML写入到文件
    with open(report_path, 'w', encoding='utf-8') as f:
        f.write(output)


# 测试数据
cases = [
    {
        'case_name': 'Test Case 1',
        'case_desc': 'This is a test case.',
        'priority': 'High',
        'mark': 'Login',
        'run_status': 200,
        'run_msg': 'Test passed.'
    },
    {
        'case_name': 'Test Case 2',
        'case_desc': 'This is another test case.',
        'priority': 'Low',
        'mark': 'Signup',
        'run_status': 500,
        'run_msg': 'Test failed.'
    },
]

# 输出报告
# render_report(cases, datetime.now(), 'report.html')



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/10 16:00:00
"""


class RequestException(Exception):
    """
    RequestException class.
    请求过程中异常
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class FormatCheckException(Exception):
    """
    FormatCheckException class.
    case格式检查异常
    """

    def __init__(self, case_name, context):
        self.message = f'case: {case_name} 格式检查失败，请检查case的格式是否正确，具体信息如下：{context}'
        super().__init__(self.message)


class FormatBTStringException(Exception):
    """
    FormatBTStringException class.
    字符串格式异常
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class BTFuncNotFoundException(Exception):
    """
    BTFuncNotFoundException class.
    bt_func方法未找到异常
    """

    def __init__(self, func_name):
        self.message = f'方法"{func_name}"不存在，请先使用@bt_func进行方法注册'
        super().__init__(self.message)


class BTActionNotFoundException(Exception):
    """
    BTActionNotFoundException class.
    bt_action方法未找到异常
    """

    def __init__(self, action_name):
        self.message = f'动作"{action_name}"不存在，请先使用@bt_action进行动作注册'
        super().__init__(self.message)


class BTAssertNotFoundException(Exception):
    """
    BTAssertNotFoundException class.
    bt_assert方法未找到异常
    """

    def __init__(self, assert_name):
        self.message = f'断言方法"{assert_name}"不存在，请先使用@bt_assert进行断言方法注册'
        super().__init__(self.message)


class BTFuncFormatCheckException(Exception):
    """
    BTFuncFormatCheckException class.
    bt_func方法格式检查异常
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class BTActionFormatCheckException(Exception):
    """
    BTActionFormatCheckException class.
    bt_action方法格式检查异常
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class BTAssertFormatCheckException(Exception):
    """
    BTAssertFormatCheckException class.
    bt_assert方法格式检查异常
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class AssertFailException(Exception):
    """
    AssertFailException class.
    断言失败异常
    """

    def __init__(self, expect_data, actual_data, assert_name):
        self.message = f'执行"{assert_name}"断言失败' \
                       f'\nexpect: {expect_data}' \
                       f'\nassert: {actual_data}'
        super().__init__(self.message)


class BTAssertDictFormatException(Exception):
    """
    BTAssertDictFormatException class.
    Assert的dict格式存在问题，可能是缺少expect、actual、operator中的某些字段
    """

    def __init__(self, assert_dict, missing_item):
        self.message = f'断言 {assert_dict} 格式存在问题，缺少字段"{missing_item}"'
        super().__init__(self.message)


class BTCaseNotLoadException(Exception):
    """
    BTCaseNotLoadException class.
    case文件不存在异常
    """
    def __init__(self):
        self.message = 'case文件为加载，请先使用bintest.case_load()加载'
        super().__init__(self.message)



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/12 19:18:00
"""
import itertools

from zjbbintest.Actuator.bin_test_exception.bin_test_exception import FormatBTStringException, BTFuncNotFoundException, \
    BTActionNotFoundException
from zjbbintest.bintest_data import BinTestData
from jsonpath_rw import parse
import logging


class StringDynamicRun:
    """
    字符串动态执行方法
    主要处理$BT[]中的conf()、func()和var()，通过run方法将字符串变为单纯的纯文本字符串
    """

    def __init__(self, source_str, case_var_dict={}, req_dict={}, resp_dict={}):
        """
        :param source_str: 源字符串
        """
        # 源字符串，类似https://$BT[conf(ip)]:$BT[conf(host)]/$BT[conf(path)]?name=$BT[func(var(rebotId))]
        self.source_str = source_str if source_str else ''
        # case执行过程中的变量都存在这个字典中
        self.case_var_dict = case_var_dict
        # case执行过程中的请求列表
        self.req_dict = req_dict
        # case执行过程中的响应列表
        self.resp_dict = resp_dict
        # 不可执行字符串list
        self.__inoperable_str_list = []
        # 可执行字符串list
        self.__operable_str_list = []
        # 执行后的字符串list
        self.__operated_str_list = []
        # 最后的结果字符串，我们需要的是这个
        self.res_str = ''

    def run(self):
        """
        执行字符串动态替换
        :return: 第一个是完全替换后的字符串，第二个是case的变量字典
        """
        if type(self.source_str) != str:
            return self.source_str, self.case_var_dict
        self.__get_bintest_string()
        self.__batch_replace()
        self.__restore_str()
        return self.res_str, self.case_var_dict

    def __get_bintest_string(self):
        """
        获取字符串
        """
        # TODO: 有问题，再看看
        # source_str_len = len(self.source_str)
        # if source_str_len < 5:
        #     raise FormatBTStringException(f'字符串"{self.source_str}"格式存在问题，请检查字符串内容')
        stack = []
        index_list = []
        try:
            # 丑陋，但是有效
            new_str = self.source_str.replace("$BT[", "…")
            new_str_len = len(new_str)
            for index in range(new_str_len):
                if new_str[index] == '[' or new_str[index] == '…':
                    stack.append(index)
                elif new_str[index] == ']':
                    left = stack.pop()
                    if new_str[left] == '…':
                        right = index
                        index_list.append((left, right))
            self.__split_str(new_str, index_list)
        except Exception as e:
            print(e)
            raise FormatBTStringException(f'字符串"{self.source_str}"格式存在问题，请检查括号[]是否匹配')

    def __batch_replace(self):
        """
        批量替换字符串
        :return:
        """
        str_dynamic_run_tool = StringDynamicRunTool(self.case_var_dict, self.req_dict, self.resp_dict)
        self.__operated_str_list = [str_dynamic_run_tool.run(operable_str)
                                    for operable_str in self.__operable_str_list]

    def __restore_str(self):
        """

        :return:
        """
        if self.__inoperable_str_list == ["", ""] and len(self.__operated_str_list) == 1:
            self.res_str = self.__operated_str_list[0]
        else:
            paired = itertools.zip_longest(self.__inoperable_str_list, self.__operated_str_list, fillvalue='')
            self.res_str = ''.join([i + (j if j is not None else '') for i, j in paired])

    def __split_str(self, new_str, index_list):
        """
        切割字符串，把要动态执行的字符串切割出来，放在一个列表中，剩余部分也放在一个列表中，这个两个列表就是返回值
        丑陋，但是有效
        :param new_str:
        :param index_list:
        :return:
        """
        mid_list = [-1, ]
        for (left, right) in index_list:
            mid_list.append(left)
            mid_list.append(right)
        mid_list.append(len(new_str))
        len_mid = len(mid_list)
        source_index_list = [(mid_list[i], mid_list[i + 1]) for i in range(0, len_mid, 2)]
        # print(source_index_list)
        # print(index_list)
        for (left, right) in index_list:
            self.__operable_str_list.append(new_str[left + 1: right])
        for (left, right) in source_index_list:
            self.__inoperable_str_list.append(new_str[left + 1: right])


class StringDynamicRunTool:
    """
    字符串动态执行工具类
    初始化时需要传入一个
    字典类型case_var_dict，代表该条case的变量
    dict类型req_dict，表示该条case的请求信息
    dict类型resp_dict，表示该条case的响应信息
    """

    def __init__(self, case_var_dict, req_dict, resp_dict):
        """
        :param case_var_dict: case级变量字典
        """
        self.case_var_dict = case_var_dict
        self.req_dict = req_dict
        self.resp_dict = resp_dict

    def run(self, operable_str):
        """
        传入可以执行的字符串如func()、var()、set()、action()，执行具体动作，返回执行后的字符串和case级变量字典
        :param operable_str:
        :return:
        """
        logging.debug(f'动态执行字符串：{operable_str}')
        if operable_str is None or operable_str == '':
            return ''
        operable_str = operable_str.strip()
        # if operable_str.startswith("conf(") and operable_str.endswith(")"):
        #     new_str = operable_str[5:-1]
        #
        #     value_str = self.run(new_str)
        #     value_path = value_str.split('.')
        #     res_str = BinTestData.config_dict
        #     for value_path_item in value_path:
        #         res_str = res_str.get(value_path_item)
        #     return res_str
        if operable_str.startswith("func(") and operable_str.endswith(")"):
            new_str = operable_str[5:-1]
            value_str = self.run(new_str)
            func_name = value_str.split('(')[0]
            func_args = value_str[len(func_name) + 1: len(value_str) - 1].split(',')
            func_args = [self.run(arg) for arg in func_args]
            try:
                if func_args is None or func_args == [] or func_args == [""]:
                    logging.debug(f'执行无参方法{func_name}')
                    res_str = BinTestData.func_dict.get(func_name)()
                else:
                    logging.debug(f'执行有参方法{func_name}，参数为:{func_args}')
                    res_str = BinTestData.func_dict.get(func_name)(*func_args)
            except TypeError:
                raise BTFuncNotFoundException(func_name)
            return res_str
        elif operable_str.startswith("var(") and operable_str.endswith(")"):
            # TODO 换成jsonpath的方式获取变量
            # DONE 2024-5-9
            new_str = operable_str[4:-1]
            value_str = self.run(new_str)
            if value_str.startswith("$var"):
                # 获取用户自定义变量
                data_source = self.case_var_dict
                json_path = '$' + value_str[4:]
                logging.debug(f'从变量中获取内容，jsonpath为:{json_path}；变量字典为:{data_source}')
            elif value_str.startswith("$req"):
                # 获取请求响应变量
                data_source = self.req_dict
                json_path = '$' + value_str[4:]
                logging.debug(f'从请求中获取内容，jsonpath为:{json_path}；请求字典为:{data_source}')
            elif value_str.startswith("$resp"):
                # 获取请求响应变量
                data_source = self.resp_dict
                json_path = '$' + value_str[5:]
                logging.debug(f'从响应中获取内容，jsonpath为:{json_path}；响应字典为:{data_source}')
            elif value_str.startswith("$conf"):
                # 获取配置文件变量
                data_source = BinTestData.config_dict
                json_path = '$' + value_str[5:]
                logging.debug(f'从配置文件中获取内容，jsonpath为:{json_path}；配置字典为:{data_source}')
            else:
                return None
            expr = parse(json_path)
            result = expr.find(data_source)
            if result:
                res_list = []
                for match in result:
                    res_list.append(match.value)
                if len(res_list) == 1:
                    return res_list[0]
                else:
                    return res_list
            else:
                return None
            # for value_path_item in value_path[1:]:
            #     if value_path_item.isdigit():
            #         try:
            #             data_source = data_source[int(value_path_item)]
            #         except IndexError:
            #             return None
            #     else:
            #         data_source = data_source.get(value_path_item)
            # return data_source
        elif operable_str.startswith("set(") and operable_str.endswith(")"):
            new_str = operable_str[4:-1]
            # 获取变量名和值
            index = new_str.find(',')
            var_key = new_str[:index]
            var_value = new_str[index + 1:]
            var_value = self.run(var_value)
            self.case_var_dict[var_key] = var_value
            logging.debug(f'设置变量{var_key}为{var_value}')
            return ''
        elif operable_str.startswith("action(") and operable_str.endswith(")"):
            new_str = operable_str[7:-1]
            value_str = self.run(new_str)
            action_name = value_str.split('(')[0]
            action_args = value_str[len(action_name) + 1: len(value_str) - 1].split(',')
            action_args = [self.run(arg) for arg in action_args]
            try:
                if action_args is None or action_args == [] or action_args == [""]:
                    logging.debug(f'执行无参动作{action_name}')
                    BinTestData.action_dict.get(action_name)()
                else:
                    logging.debug(f'执行有参动作{action_name}，参数为{action_args}')
                    BinTestData.action_dict.get(action_name)(*action_args)
            except TypeError as e:
                raise BTActionNotFoundException(action_name)
            return ''
        else:
            return operable_str



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/5/8 16:56:00
"""
import json
from urllib.parse import urlparse, parse_qs


def read_har(har_file_path):
    """
    读取har文件，返回json格式字符串
    :param har_file_path: har文件路径
    :return: json格式字符串
    """
    with open(har_file_path, 'r', encoding='utf-8') as f:
        return f.read()


# def analysis(har_file_path, res_file_path=None):
#     """
#     解析har文件，生成测试用例的json格式
#     :param har_file_path: har文件路径
#     :param res_file_path: 解析结果文件路径，如果为None，返回json格式字符串
#     """
#     with open(har_file_path, 'r', encoding='utf-8') as f:
#         har_json = f.read()
#     return analysis_json(har_json)


def analysis_har_list(har_list):
    """
    解析har文件列表，生成测试用例的json格式
    :param har_list:
    :return:
    """
    json_output = {
        "caseName": "",
        "description": "",
        "priority": 0,
        "testSteps": []
    }
    index = 0
    # 提取 request 信息
    for _, entry in enumerate(har_list):
        request = entry['request']
        parsed_url = urlparse(request['url'])
        url = parsed_url.scheme + '://' + parsed_url.netloc
        path = parsed_url.path
        params = {k: v[0] for k, v in parse_qs(parsed_url.query).items()}
        body = request.get('postData', {})
        if 'json' in body.get('mimeType', ''):  # 检查MIME类型是否为JSON
            body = json.loads(body.get('text', '{}'))  # 是则解析
        step = {
            "step": index,
            "request": {
                "url": url,
                "path": path,
                "method": request['method'],
                "headers": {header['name']: header['value'] for header in request['headers']},
                "params": params,
                "body": body
            },
            "action": [],
            "assert": [
                {
                    "expect": 200,
                    "actual": f"$BT[var($resp['{index}'].status_code)]",
                    "operator": "=="
                }
            ]
        }
        json_output['testSteps'].append(step)
        index += 1

    return json_output


# def analysis_json(har_json):
#     """
#     解析har文件，生成测试用例的json格式
#     :param har_json:
#     :return:
#     """
#
#     har_dict = json.loads(har_json)
#     json_output = {
#         "caseName": "",
#         "description": "",
#         "priority": 0,
#         "testSteps": []
#     }
#
#     index = 0
#     # 提取 request 信息
#     for _, entry in enumerate(har_dict['log']['entries']):
#         if entry.get('_resourceType') != 'xhr':
#             continue
#         request = entry['request']
#         parsed_url = urlparse(request['url'])
#         url = parsed_url.scheme + '://' + parsed_url.netloc
#         path = parsed_url.path
#         params = {k: v[0] for k, v in parse_qs(parsed_url.query).items()}
#         body = request.get('postData', {})
#         if 'json' in body.get('mimeType', ''):  # 检查MIME类型是否为JSON
#             body = json.loads(body.get('text', '{}'))  # 是则解析
#         step = {
#             "step": index,
#             "request": {
#                 "url": url,
#                 "path": path,
#                 "method": request['method'],
#                 "headers": {header['name']: header['value'] for header in request['headers']},
#                 "param": params,
#                 "body": body
#             },
#             "action": [],
#             "assert": [
#                 {
#                     "expect": "",
#                     "actual": "",
#                     "operator": ""
#                 }
#             ]
#         }
#         json_output['testSteps'].append(step)
#         index += 1
#
#     return json_output
#     # # 输出结果
#     # res_str = json.dumps(json_output, indent=4)
#     # if res_file_path:
#     #     with open(res_file_path, 'w', encoding='utf-8') as f:
#     #         f.write(res_str)
#     #     return res_str
#     # else:
#     #     return res_str


def parse_har_list(har_list):
    """
    解析HAR文件列表，返回一个字典，包含请求和响应的键值对
    :param har_list: HAR文件列表
    :return: 请求和响应的字典
    """
    # 初始化请求和响应字典
    req_dict = {}
    resp_dict = {}

    for entry in har_list:
        request = entry['request']
        response = entry['response']
        request_id = str(len(req_dict))

        # 提取请求的URL和路径
        url = request['url']
        parsed_url = urlparse(url)
        path = parsed_url.path

        # 提取请求方法
        method = request['method']

        # 提取请求头
        headers = request['headers']
        headers_dict = {}
        for header in headers:
            headers_dict[header['name']] = header['value']

        # 提取查询参数
        params = {}
        if 'queryString' in request:
            for param in request['queryString']:
                params[param['name']] = param['value']

        # 提取请求体
        body = {}
        if 'postData' in request:
            post_data = request['postData']
            if 'text' in post_data:
                try:
                    body = json.loads(post_data['text'])
                except json.JSONDecodeError:
                    body = post_data['text']

        # 提取响应状态码和文本
        status_code = response['status']
        text = response['content']['text'] if 'content' in response and 'text' in response['content'] else None
        if text is None:
            json_data = None
        else:
            try:
                json_data = json.loads(text)
            except json.JSONDecodeError:
                json_data = text

        # 将提取的信息添加到字典中
        req_dict[request_id] = {
            'url': parsed_url.netloc,
            'path': path,
            'method': method,
            'headers': headers_dict,
            'params': params,
            'body': body
        }

        resp_dict[request_id] = {
            'status_code': status_code,
            'text': json_data
        }

    return req_dict, resp_dict


# def parse_har_file(har_file_path):
#     """
#     解析HAR文件，返回一个字典，包含请求和响应的键值对
#     :param har_file_path: HAR文件路径
#     :return: 请求和响应的字典
#     """
#     # 读取HAR文件
#     with open(har_file_path, 'r') as file:
#         har_data = json.load(file)
#
#         # 初始化请求和响应字典
#         req_dict = {}
#         resp_dict = {}
#
#         # 遍历HAR文件中的每个条目
#         for entry in har_data['log']['entries']:
#             if entry.get('_resourceType') != 'xhr':
#                 continue
#             request = entry['request']
#             response = entry['response']
#             request_id = str(len(req_dict))
#
#             # 提取请求的URL和路径
#             url = request['url']
#             parsed_url = urlparse(url)
#             path = parsed_url.path
#
#             # 提取请求方法
#             method = request['method']
#
#             # 提取请求头
#             headers = request['headers']
#             headers_dict = {}
#             for header in headers:
#                 headers_dict[header['name']] = header['value']
#
#             # 提取查询参数
#             params = {}
#             if 'queryString' in request:
#                 for param in request['queryString']:
#                     params[param['name']] = param['value']
#
#             # 提取请求体
#             body = {}
#             if 'postData' in request:
#                 post_data = request['postData']
#                 if 'text' in post_data:
#                     try:
#                         body = json.loads(post_data['text'])
#                     except json.JSONDecodeError:
#                         body = post_data['text']
#
#             # 提取响应状态码和文本
#             status_code = response['status']
#             text = response['content']['text'] if 'content' in response and 'text' in response['content'] else None
#             if text is None:
#                 json_data = None
#             else:
#                 try:
#                     json_data = json.loads(text)
#                 except json.JSONDecodeError:
#                     json_data = text
#
#             # 将提取的信息添加到字典中
#             req_dict[request_id] = {
#                 'url': parsed_url.netloc,
#                 'path': path,
#                 'method': method,
#                 'headers': headers_dict,
#                 'params': params,
#                 'body': body
#             }
#
#             resp_dict[request_id] = {
#                 'status_code': status_code,
#                 'text': json_data
#             }
#
#     return req_dict, resp_dict


def save_json(dict_data, file_path):
    """
    将数据保存为JSON文件
    :param dict_data: 数据
    :param file_path: 文件路径
    :return: None
    """
    res_str = json.dumps(dict_data, indent=4, ensure_ascii=False)
    if file_path:
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(res_str)


def have_easy_har(har_file_path):
    """
    获取简化的har内容，通过交互式操作，选出要保留的请求和响应
    :param har_file_path:
    :return:
    """
    # 读取HAR文件
    with open(har_file_path, 'r') as file:
        har_data = json.load(file)

    easy_json_list = []
    index = 0
    for entry in har_data['log']['entries']:
        if entry.get('_resourceType') != 'xhr':
            continue
        index += 1
        url = entry['request']['url']
        pass_flag = False
        while pass_flag is False:
            user_input = input(f'第{index}个接口  {url} 是否需要保留？y/n:')
            if user_input.upper() in ['Y', 'YES']:
                easy_json_list.append(entry)
                pass_flag = True
            elif user_input.upper() in ['N', 'NO']:
                pass_flag = True
            else:
                print('输入错误，请重新输入，y表示保留，n表示舍弃')
    return easy_json_list


if __name__ == '__main__':
    # analysis('/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/机器人搜索.har',
    #          '/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/机器人搜索.json')
    # req_dict, resp_dict = parse_har_file(
    #     '/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/机器人搜索.har')
    # print(json.dumps(req_dict))
    # print(json.dumps(resp_dict))
    print(json.dumps(
        have_easy_har('/Users/zhangjiabin01/Desktop/new_auto/baidu/kefu-qa/bintest/har/通知机器人测一测.har')))



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/20 19:02:00
"""
from zjbbintest.Actuator.utils.string_dynamic_run import StringDynamicRun


class BatchActionExecutor:
    """
    Batch action executor.
    批量动作执行器
    """

    def __init__(self, action_list):
        """
        Init batch action executor.
        初始化批量动作执行器
        :param action_list: 动作列表
        :type action_list: list
        """
        self.__action_list = action_list

    def execute(self, case_var_dict, req_dict, resp_dict):
        """
        Execute batch action.
        执行批量动作
        :return: 返回结果
        :rtype: list
        """
        for action in self.__action_list:
            _, case_var_dict = StringDynamicRun(action, case_var_dict, req_dict, resp_dict).run()
        return case_var_dict



"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/22 17:41:21
"""
import os


def get_json_files_in_directories(path_list):
    # 初始化一个list来存储所有.json文件的绝对路径
    file_list = []

    for path in path_list:
        # 判断path是不是文件，如果是则直接添加到list
        if os.path.isfile(path) and os.path.splitext(path)[1] == '.json':
            file_list.append(path)
            continue

        # 使用os.walk遍历目录下所有子文件和子文件夹
        for dir_path, dir_names, filenames in os.walk(path):
            for filename in filenames:
                # 只有当文件是.json文件时，才添加文件的绝对路径到list中
                if os.path.splitext(filename)[1] == '.json':
                    file_list.append(os.path.join(dir_path, filename))

    return file_list

if __name__ == '__main__':
    path_list = ['/Users/zhangjiabin01/Downloads']
    print(get_json_files_in_directories(path_list))



from setuptools import setup, find_packages

dependencies = []

# 打开requirements.txt文件并读取内容
with open('requirements.txt', 'r') as file:
    for line in file:
        dependencies.append(line.strip())

print(dependencies)

setup(
    name='zjbbintest',
    version='2.7',
    author="zhangjiabin01",
    author_email="zhangjiabin01@baidu.com",
    description="bintest自动化框架",
    packages=find_packages(),
    package_data={
            'zjbbintest': ['Template/*'],  # 包含my_package中的templates文件夹下的所有文件
        },
    install_requires=dependencies,
)


"""
Copyright (c) 2024 Baidu.com, Inc. All Rights Reserved
This module provide configigure file management service in i18n environment.

Authors: zhangjiabin01
Date: 2024/4/12 13:19:00
"""
import configparser
from datetime import datetime
import json
import os
from functools import wraps
from inspect import getmembers, isfunction

from zjbbintest.Actuator.bin_test_case.bin_test_case import BinTestCase
from zjbbintest.Actuator.bin_test_exception.bin_test_exception import BTFuncFormatCheckException, \
    BTAssertFormatCheckException, \
    BTActionFormatCheckException, BTCaseNotLoadException
from zjbbintest.Actuator.utils.processing_path import get_json_files_in_directories
from zjbbintest.Analysis.har2json import save_json, have_easy_har, analysis_har_list, \
    parse_har_list
from zjbbintest.Analysis.replace_values import get_req_bt_str_map, get_resp_bt_str_map, save_conf_var, \
    get_conf_bt_str_map, case_json_replace_bintest_var
from zjbbintest.Template.bintest_template import render_report
from zjbbintest.bintest_data import BinTestData
import logging


# config_dict = {}
# func_dict = {}
# assert_dict = {}
# action_dict = {}
# run_case_list = []
# case_load_flag = False


def bt_func(*args):
    """
    装饰器，进行函数入参和返回值的判断，入参个数必须与args的长度相等，且类型必须与args对应
    :param args:
    :return:
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*func_args, **func_kwargs):
            converted_args = []
            all_args = list(func_args) + list(func_kwargs.values())
            if len(args) != len(all_args):
                raise BTFuncFormatCheckException(
                    f'{func.__name__}方法需要{len(args)}个参数，实际传入{len(all_args)}个参数')

            for a, b in zip(args, all_args):
                try:
                    converted_args.append(a(b))
                except Exception:
                    raise BTFuncFormatCheckException(f'{func.__name__}方法的参数 {b} 无法转为 {a.__name__} 类型')
            result = func(*converted_args, **func_kwargs)
            if result is None:
                raise BTFuncFormatCheckException(f'{func.__name__}方法必须有返回值')
            return result

        wrapper.is_bt_func = True
        return wrapper

    return decorator


def bt_action(*args):
    """
    装饰器，进行函数入参和返回值的判断，入参个数必须与args的长度相等，且类型必须与args对应，无返回值
    :param args:
    :return:
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*func_args, **func_kwargs):
            converted_args = []
            all_args = list(func_args) + list(func_kwargs.values())
            if len(args) != len(all_args):
                raise BTActionFormatCheckException(
                    f'{func.__name__}方法需要{len(args)}个参数，实际传入{len(all_args)}个参数')

            for a, b in zip(args, all_args):
                try:
                    converted_args.append(a(b))
                except Exception:
                    raise BTFuncFormatCheckException(f'{func.__name__}方法的参数 {b} 无法转为 {a.__name__} 类型')
            # for i, (a, b) in enumerate(zip(args, all_args)):
            #     if not isinstance(b, a):
            #         raise BTActionFormatCheckException(
            #             f'{func.__name__}方法的第{i + 1}个参数类型应为{a.__name__}，实际为{type(b).__name__}')
            result = func(*converted_args, **func_kwargs)
            if result is not None:
                raise BTActionFormatCheckException(f'{func.__name__}方法必须无返回值')
            return result

        wrapper.is_bt_action = True
        return wrapper

    return decorator


def bt_assert(func):
    """
    装饰器，进行函数入参和返回值的判断，入参个数为2个，返回值必须为boolean类型
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        # 检查入参个数
        arg_count = len(args) + len(kwargs)
        if arg_count != 2:
            raise BTAssertFormatCheckException(f"{func.__name__}方法需要2个参数，实际传入{arg_count}个参数")

        # 执行函数并获取返回值
        result = func(*args, **kwargs)

        # 检查返回值是否为boolean类型
        if not isinstance(result, bool):
            raise BTAssertFormatCheckException(f"{func.__name__}方法返回值必须为boolean类型")

        return result

    # 标记为bt_assert
    func.is_bt_assert = True
    return wrapper


def conf_load(path):
    """
    读取配置文件，将配置文件中的内容存进config_dict中，以便全局使用
    path是个.ini文件，里面是配置的一些全局可以使用到的变量
    :param path:
    :return:
    """
    config = configparser.ConfigParser()
    config.read(path)
    for section in config.sections():
        BinTestData.config_dict[section] = {}
        for key, value in config.items(section):
            BinTestData.config_dict[section][key] = value


def func_load(module):
    """
    根据模块名，将用户定义的函数加载到func_dict中，以便全局使用
    :param module:
    :return:
    """
    for name, obj in getmembers(module):
        if isfunction(obj) and getattr(obj, 'is_bt_func', False):
            BinTestData.func_dict[name] = obj


def action_load(module):
    """
    根据模块名，将用户定义的action加载到func_dict中，以便全局使用
    :param module:
    :return:
    """
    for name, obj in getmembers(module):
        if isfunction(obj) and getattr(obj, 'is_bt_action', False):
            BinTestData.action_dict[name] = obj


def assert_load(module):
    """
    根据模块名，将用户定义的断言函数加载到assert_dict中，以便全局使用
    :param module:
    :return:
    """
    for name, obj in getmembers(module):
        if isfunction(obj) and getattr(obj, 'is_bt_assert', False):
            BinTestData.assert_dict[name] = obj


def case_load(path_list, tags=[], priority=[]):
    """
    加载case，只会加载制定case
    :param path_list: 路径列表，支持多个路径，递归加载路径下的所有case
    :param tags: 加载指定标签的case，支持多个标签，比如['label1', 'label2']代表加载label1和label2的case
    :param priority: 加载指定优先级的case，支持多个优先级，比如[1, 2]代表加载1和2的case
    :return: 返回case列表，list中存的是BinTestCase对象
    """
    # 获取case文件
    case_path_list = get_json_files_in_directories(path_list)
    for case_path in case_path_list:
        with open(case_path, 'r') as json_file:
            data = json.load(json_file)
        if not tags and not priority:
            BinTestData.run_case_list.append(BinTestCase.create_obj_by_dict(data))
        elif tags and priority:
            case_mark_labels = data.get('tags', [])
            case_priority = data.get('priority', 999)
            if set(case_mark_labels) & set(tags) and case_priority in priority:
                BinTestData.run_case_list.append(BinTestCase.create_obj_by_dict(data))
        elif tags and not priority:
            case_mark_labels = data.get('tags', [])
            if set(case_mark_labels) & set(tags):
                BinTestData.run_case_list.append(BinTestCase.create_obj_by_dict(data))
        else:
            case_priority = data.get('priority', 999)
            if case_priority in priority:
                BinTestData.run_case_list.append(BinTestCase.create_obj_by_dict(data))
    # 修改状态
    BinTestData.case_load_flag = True
    return


def run(log_path, report_path):
    """
    执行测试用例，生成测试报告
    :param report_path: 测试报告路径
    :param log_path: 日志路径
    :return:
    """
    now_time = datetime.now()
    # 初始化日志
    if not os.path.exists(log_path):
        # 创建文件所在的目录
        dir_path = os.path.dirname(log_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        # 创建文件
        with open(log_path, 'w') as f:
            pass
        print(f"文件 {log_path} 已创建。")
    else:
        pass
    logging.basicConfig(filename=log_path, filemode='a', level=logging.DEBUG,
                        format='%(asctime)s - %(pathname)s - %(lineno)s - %(message)s')
    case_res_list = []
    if BinTestData.case_load_flag:
        # 执行测试用例
        for bin_case in BinTestData.run_case_list:
            case_res = bin_case.run()
            case_res_list.append(case_res)
        # 生成测试报告
        render_report(case_res_list, now_time, report_path)
    else:
        raise BTCaseNotLoadException()


def rebuild(har_file_path, conf_ini_file_path='', case_dir_path=''):
    """
    根据har_file_path重建case，将其放在case_dir_path目录下
    case_dir_path目录下，应该存放三个文件
    1 替换后的case的json形式 xxx_case.json
    2 变量的bt_str映射关系 xxx_bt_str.json
    3 提取的请求和响应信息 xxx_req_resp.json
    :param har_file_path:
    :param conf_ini_file_path
    :param case_dir_path:
    :return: 
    """
    # 判断har_file_path是否存在
    if not os.path.exists(har_file_path):
        raise FileNotFoundError(f"har文件 {har_file_path} 不存在。")
    if conf_ini_file_path != '':
        if not os.path.exists(conf_ini_file_path):
            raise FileNotFoundError(f"配置文件 {conf_ini_file_path} 不存在。")
        conf_dict = save_conf_var(conf_ini_file_path)
    else:
        conf_dict = {}

    # 获取文件名
    file_name = os.path.basename(har_file_path)
    # 获取路径
    dir_name = os.path.dirname(har_file_path)
    # 获取基础名（不含后缀）
    base_name, _ = os.path.splitext(file_name)

    easy_har_list = have_easy_har(har_file_path)
    case_json = analysis_har_list(easy_har_list)
    req_dict, resp_dict = parse_har_list(easy_har_list)
    req_bt_str_map = get_req_bt_str_map(req_dict)
    resp_bt_str_map = get_resp_bt_str_map(resp_dict)
    conf_bt_str_map = get_conf_bt_str_map(conf_dict)
    new_case_json = case_json_replace_bintest_var(case_json, conf_bt_str_map, req_bt_str_map, resp_bt_str_map)
    req_resp_json = {}
    for req_key, req in req_dict.items():
        req_resp_json[req_key] = {
            'request': req,
            'response': resp_dict.get(req_key)
        }
    bt_str_map_json = {
        'conf': conf_bt_str_map,
        'req': req_bt_str_map,
        'resp': resp_bt_str_map
    }
    if not case_dir_path:
        # 如果case_dir_path为空，则放在当前目录下的case目录下
        case_dir_path = dir_name + f'/{base_name}'
    case_full_path = case_dir_path + f'/{base_name}_case.json'
    bt_str_map_full_path = case_dir_path + f'/{base_name}_bt_str.json'
    req_resp_full_path = case_dir_path + f'/{base_name}_req_resp.json'
    # 创建case_dir_path目录
    if not os.path.exists(case_dir_path):
        os.makedirs(case_dir_path)
    # 保存
    save_json(new_case_json, case_full_path)
    save_json(bt_str_map_json, bt_str_map_full_path)
    save_json(req_resp_json, req_resp_full_path)





